/*
 * Copyright 2011 Hanson Robokind LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.robokind.impl.motion.dynamixel.properties;

import org.robokind.api.common.types.Temperature;
import org.robokind.api.motion.joint_properties.ReadTemperature;
import org.robokind.impl.motion.dynamixel.DynamixelServo;

/**
 *
 * @author Matthew Stevenson <www.robokind.org>
 */
public class TemperatureProperty extends ReadTemperature { 
    private DynamixelServo myDynamixelServo;
    protected Temperature myCachedValue;
    
    public TemperatureProperty(DynamixelServo dyna){
        if(dyna == null){
            throw new NullPointerException();
        }
        myDynamixelServo = dyna;
    }

    @Override
    public Temperature getValue() {
        Temperature old = myCachedValue;
        myCachedValue = readValue();
        firePropertyChange(getPropertyName(), old, myCachedValue);
        return myCachedValue;
    }
    
    private Temperature readValue(){
        Integer temperature = myDynamixelServo.getCurrentTemperature();
        if(temperature != null){
            return new Temperature(temperature);
        }
        return null;
    }

    @Override
    public Temperature getCachedValue() {
        return myCachedValue;
    }
}
